
#include <windows.h>
#include <shellapi.h>
#include "SizeSelect_Dialog.hpp"
#include "resource.h"
#include <string.h>
#include <stdio.h>
#include "registry.h"
#include "VideoResizer_Filter.hpp"

//
// create an instance
//

CUnknown*
SizeSelect_Dialog::CreateInstance (LPUNKNOWN owner, HRESULT* result)
{
	CUnknown* instance = new SizeSelect_Dialog (owner, result);
	if (instance == 0)
	{
		*result = E_OUTOFMEMORY;
	}
	return instance;
}

//
// default creation
//
SizeSelect_Dialog::SizeSelect_Dialog (LPUNKNOWN owner, HRESULT* result)
	:	CBasePropertyPage (NAME ("SizeSelect Dialog"), owner, IDD_SIZE_SELECT, IDS_ABOUT_TITLE)
{
}

//
// Called when a message is sent to the property page dialog box. 
//
BOOL SizeSelect_Dialog::OnReceiveMessage (HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	char string [1024];

	SizeSelect_Dialog* dialog = (SizeSelect_Dialog*)(GetWindowLong (window, GWL_USERDATA));
	switch (msg)
	{
		case WM_INITDIALOG:
			//dialog = (SizeSelect_Dialog*)(lParam);
			//ialog->window = window;
			//SetWindowLong (window, GWL_USERDATA, (LONG)(dialog));

			CheckDlgButton(m_Dlg, IDC_FLIP, registry_config.options.flip ? BST_CHECKED : BST_UNCHECKED);

			CheckDlgButton(m_Dlg, IDC_VCDFIX, registry_config.options.vcdfix ? BST_CHECKED : BST_UNCHECKED);

			CheckDlgButton(m_Dlg, IDC_ASPECT, (registry_config.options.keep_aspect&0x80) ? BST_CHECKED : BST_UNCHECKED);

			switch ( registry_config.options.keep_aspect&0x7f ) {
			case 0:
				SendDlgItemMessage( m_Dlg, IDC_43, BM_SETCHECK, BST_CHECKED ,0);
				SendDlgItemMessage( m_Dlg, IDC_169, BM_SETCHECK, BST_UNCHECKED ,0);
				break;
			case 1:
				SendDlgItemMessage( m_Dlg, IDC_43, BM_SETCHECK, BST_UNCHECKED ,0);
				SendDlgItemMessage( m_Dlg, IDC_169, BM_SETCHECK, BST_CHECKED ,0);
				break;
			}

			CheckDlgButton(m_Dlg, IDC_CROP, registry_config.options.crop ? BST_CHECKED : BST_UNCHECKED);

			sprintf (string, "%d", registry_config.options.crop_top );
			SetDlgItemText (hwnd, IDC_CTOP, string);
			sprintf (string, "%d", registry_config.options.crop_bottom );
			SetDlgItemText (hwnd, IDC_CBOT, string);

			sprintf (string, "%d", registry_config.options.width );
			SetDlgItemText (hwnd, IDC_WIDTH, string);
			sprintf (string, "%d", registry_config.options.height );
			SetDlgItemText (hwnd, IDC_HEIGHT, string);

			sprintf (string, "%.3f", registry_config.options.b );
			SetDlgItemText (hwnd, IDC_BICUBIC_B, string);
			sprintf (string, "%.3f", registry_config.options.c );
			SetDlgItemText (hwnd, IDC_BICUBIC_C, string);

			sprintf (string, "%d x %d", input_width, input_height);
			SetDlgItemText (hwnd, IDC_INCOMING_RESOLUTION, string);
			return TRUE;

		case WM_COMMAND:

			switch(HIWORD(wParam))
			{
				case BN_CLICKED:
				{
					switch(LOWORD(wParam))
					{
						case IDC_FLIP:
							registry_config.options.flip = !!IsDlgButtonChecked(m_Dlg, IDC_FLIP);
							break;
						case IDC_VCDFIX:
							registry_config.options.vcdfix = !!IsDlgButtonChecked(m_Dlg, IDC_VCDFIX);
							break;
						case IDC_ASPECT:
							if ( IsDlgButtonChecked(m_Dlg, IDC_ASPECT) ) {
								registry_config.options.keep_aspect |= 0x80;
							} else {
								registry_config.options.keep_aspect &= 0x7f;
							}
							break;
						case IDC_CROP:
							registry_config.options.crop = !!IsDlgButtonChecked(m_Dlg, IDC_CROP);
							break;
						case IDC_43:
							SendDlgItemMessage( m_Dlg, IDC_43, BM_SETCHECK, BST_CHECKED ,0);
							SendDlgItemMessage( m_Dlg, IDC_169, BM_SETCHECK, BST_UNCHECKED ,0);
							registry_config.options.keep_aspect = (registry_config.options.keep_aspect&0x80);
							break;
						case IDC_169:
							SendDlgItemMessage( m_Dlg, IDC_43, BM_SETCHECK, BST_UNCHECKED ,0);
							SendDlgItemMessage( m_Dlg, IDC_169, BM_SETCHECK, BST_CHECKED ,0);
							registry_config.options.keep_aspect = 1 + (registry_config.options.keep_aspect&0x80);
							break;
						case IDC_SAVECONFIG:
						{
							GetDlgItemText ( hwnd, IDC_WIDTH, string, 1023);
							registry_config.options.width = atoi (string);
	
							GetDlgItemText ( hwnd, IDC_HEIGHT, string, 1023);
							registry_config.options.height = atoi (string);

							GetDlgItemText ( hwnd, IDC_BICUBIC_B, string, 1023);
							registry_config.options.b = atof (string);
	
							GetDlgItemText ( hwnd, IDC_BICUBIC_C, string, 1023);
							registry_config.options.c = atof (string);
							
							registry_config.options.flip = !!IsDlgButtonChecked(m_Dlg, IDC_FLIP);

							registry_config.options.vcdfix = !!IsDlgButtonChecked(m_Dlg, IDC_VCDFIX);

							if ( IsDlgButtonChecked(m_Dlg, IDC_ASPECT) ) {
								if ( IsDlgButtonChecked(m_Dlg, IDC_43) ) {
									registry_config.options.keep_aspect = 0x80;
								} else {
									registry_config.options.keep_aspect = 0x81;
								}
							} else {
								registry_config.options.keep_aspect = registry_config.options.keep_aspect&0x7f;
							}
							
							registry_config.options.crop = !!IsDlgButtonChecked(m_Dlg, IDC_CROP);
							GetDlgItemText ( hwnd, IDC_CTOP, string, 1023);
							registry_config.options.crop_top = atof (string);
							GetDlgItemText ( hwnd, IDC_CBOT, string, 1023);
							registry_config.options.crop_bottom = atof (string);

							SaveRegistryConfig( &registry_config );
							break;
						}
						case IDC_NTSC:
						{
							sprintf (string, "%d", 640 );
							SetDlgItemText (hwnd, IDC_WIDTH, string);
							sprintf (string, "%d", 480 );
							SetDlgItemText (hwnd, IDC_HEIGHT, string);
							break;
						}
						case IDC_PAL:
						{
							sprintf (string, "%d", 768 );
							SetDlgItemText (hwnd, IDC_WIDTH, string);
							sprintf (string, "%d", 576 );
							SetDlgItemText (hwnd, IDC_HEIGHT, string);
							break;
						}
					}
					break;
				}
			}
		break;
	}

	return this->CBasePropertyPage::OnReceiveMessage (hwnd, msg, wParam, lParam);
}
/*

#include <string.h>

//
// create with specified incoming width/height
//
SizeSelect_Dialog::SizeSelect_Dialog (int width, int height)
	:	incoming_width (width),
		incoming_height (height),
		scaleTo_width (width),
		scaleTo_height (height),
		upSampling (SAMPLING_NEAREST),
		downSampling (SAMPLING_NEAREST),
		window (0)
{
}



//
// create the dialog
//
void
SizeSelect_Dialog::createDialog (HINSTANCE instance)
{
	DialogBoxParam (instance, MAKEINTRESOURCE (IDD_SIZE_SELECT), HWND_DESKTOP, SizeSelect_Dialog::DialogProc, (LPARAM)(this));
}


//
// on WM_INITDIALOG
//
void
SizeSelect_Dialog::on_initDialog ()
{
	char string [1024];

	CheckRadioButton (this->window, IDC_DOWN_NEAREST, IDC_DOWN_CUBIC, IDC_DOWN_NEAREST);
	CheckRadioButton (this->window, IDC_UP_NEAREST,		IDC_UP_CUBIC,		IDC_UP_NEAREST);

	sprintf (string, "%d", 640 );
	SetDlgItemText (this->window, IDC_WIDTH, string);
	sprintf (string, "%d", 480 );
	SetDlgItemText (this->window, IDC_HEIGHT, string);

	sprintf (string, "%d x %d", this->incoming_width, this->incoming_height);
	SetDlgItemText (this->window, IDC_INCOMING_RESOLUTION, string);
}


//
// on WM_CLOSE
//
void
SizeSelect_Dialog::on_close ()
{
	char string [1024];
	
	GetDlgItemText (this->window, IDC_WIDTH, string, 1023);
	this->scaleTo_width = atoi (string);

	GetDlgItemText (this->window, IDC_HEIGHT, string, 1023);
	this->scaleTo_height = atoi (string);
}



//
// dialog box procedure
//
BOOL CALLBACK
SizeSelect_Dialog::DialogProc (HWND window, UINT message, WPARAM wParam, LPARAM lParam)
{
	SizeSelect_Dialog* dialog = (SizeSelect_Dialog*)(GetWindowLong (window, GWL_USERDATA));
	switch (message)
	{
		case WM_INITDIALOG:
			dialog = (SizeSelect_Dialog*)(lParam);
			dialog->window = window;
			SetWindowLong (window, GWL_USERDATA, (LONG)(dialog));
			dialog->on_initDialog ();
		return TRUE;

		case WM_CLOSE:
			dialog->on_close ();
			EndDialog (window, 0);
		return TRUE;

		case WM_COMMAND:
			switch (wParam)
			{
				case IDOK:
					PostMessage (window, WM_CLOSE, 0, 0);
				return TRUE;
			}
		break;
	}
	return FALSE;
}
*/