
#include <windows.h>
#include <initguid.h>
#include "VideoResizer.hpp"
#include "VideoResizer_Filter.hpp"
#include "SizeSelect_Dialog.hpp"
#include "streams.h"
#include "registry.h"
#include "Resizer_YUY2\resizer_yuy2.h"
#include "vfw.h"

// Other data
int input_width = 0, input_height = 0;
int output_width = 0, output_height = 0;
//int c_width = 0, c_height = 0;
TConfig	registry_config;

// not defined (currently) in the directshow headers
// remove this structure if it clashes in the future
typedef struct tagVIDEOINFOHEADER2 {
    RECT                rcSource;
    RECT                rcTarget;
    DWORD               dwBitRate;
    DWORD               dwBitErrorRate;
    REFERENCE_TIME      AvgTimePerFrame;
    DWORD               dwInterlaceFlags;
    DWORD               dwCopyProtectFlags;
    DWORD               dwPictAspectRatioX; 
    DWORD               dwPictAspectRatioY; 
    DWORD               dwReserved1;        
    DWORD               dwReserved2;        
    BITMAPINFOHEADER    bmiHeader;
} VIDEOINFOHEADER2;

REFERENCE_TIME temp_AvgTimePerFrame;

#pragma warning(disable:4355)	// disable 'this in constructor list' warning


#define CHECK_RESULT \
	if (result != S_OK) \
	{ \
		/*MessageBox (HWND_DESKTOP, "result != S_OK", "", MB_OK);*/ \
		return result; \
	}


//
// construction
//
VideoResizer_Filter::VideoResizer_Filter (LPUNKNOWN owner, HRESULT* result)
	:	CTransformFilter (NAME ("Video Resizer Base Filter"), owner, CLSID_VideoResizer )
{
	LoadRegistryConfig( &registry_config );
}
	

//
// create an instance
//
CUnknown*
VideoResizer_Filter::CreateInstance (LPUNKNOWN owner, HRESULT* result)
{
	CUnknown* instance = new VideoResizer_Filter (owner, result);
	if (instance == 0)
	{
		*result = E_OUTOFMEMORY;
	}
	return instance;
}

STDMETHODIMP VideoResizer_Filter::get_prefs( int fun )
{
	return 0;
}

STDMETHODIMP VideoResizer_Filter::set_prefs( int fun )
{
	return 0;
}

//
// query interface
//
STDMETHODIMP
VideoResizer_Filter::NonDelegatingQueryInterface (REFIID riid, void** object)
{
	CAutoLock autoLock (&this->m_csFilter);

	if (riid == CLSID_VideoResizer)
	{
		return GetInterface ( (IVideoResizer*)this, object);
	} else if (riid == IID_ISpecifyPropertyPages)
	{
		return GetInterface (static_cast<ISpecifyPropertyPages*> (this), object);
	}
	else
	{
    return this->CTransformFilter::NonDelegatingQueryInterface (riid, object);
  }
}

int FindMyself(IFilterGraph* pGB)
{
    HRESULT hr;

    IEnumFilters* pEnum = NULL;
    IBaseFilter* pFilter = NULL;
	IVideoResizer *pDVF = NULL;

    hr = pGB->EnumFilters(&pEnum);
    if(FAILED(hr)) return(0);

	int nInstances = 0;

    while(pEnum->Next(1, &pFilter, NULL) == S_OK)
    {
        hr = pFilter->QueryInterface(CLSID_VideoResizer, (LPVOID*)&pDVF);

        pFilter->Release();

        if(FAILED(hr)) continue;

		nInstances++;

		pDVF->Release();
    }

    pEnum->Release();

    return(nInstances);
}

HRESULT VideoResizer_Filter::JoinFilterGraph(IFilterGraph* pGraph, LPCWSTR pName)
{
	if(pGraph)
	{
		if(FindMyself(pGraph) > 1) 
		{
			return E_FAIL;
		}
	}
	else
	{
	}

//	if( pGraph ) {
//		if ( filter_loaded == true ) return E_FAIL;
//		filter_loaded = true;
//	}

	return CTransformFilter::JoinFilterGraph(pGraph, pName);
}

//
// Verifies that the input pin supports the media type.
//
HRESULT VideoResizer_Filter::CheckInputType (const CMediaType* mediaType)
{
	// assertion
	CheckPointer (mediaType, E_POINTER);

	// major type supported?
	if (!IsEqualGUID (*mediaType->Type (), MEDIATYPE_Video))
	{
		return E_FAIL;
	}

	// minor type supported?
	if ( (!IsEqualGUID (*mediaType->Subtype (), MEDIASUBTYPE_YUY2))) {
		return E_FAIL;
	}

	// format type supported
	if ((!IsEqualGUID (*mediaType->FormatType (), FORMAT_VideoInfo)) &&
			(!IsEqualGUID (*mediaType->FormatType (), FORMAT_VideoInfo2)))
	{
		return E_FAIL;
	}

	// we can support this connection
	return S_OK;
}


//
// Verifies that the output pin supports the media type (not an override)
//
HRESULT VideoResizer_Filter::CheckOutputType (const CMediaType* mediaType)
{
	// assertion
	CheckPointer (mediaType, E_POINTER);

	// major type supported?
	if (!IsEqualGUID (*mediaType->Type (), MEDIATYPE_Video))
	{
		return E_FAIL;
	}

	// minor type supported?
	if ( !IsEqualGUID (*mediaType->Subtype (), MEDIASUBTYPE_YUY2) )
	{
		return E_FAIL;
	}
	// format type supported
	if ((!IsEqualGUID (*mediaType->FormatType (), FORMAT_VideoInfo)) &&
			(!IsEqualGUID (*mediaType->FormatType (), FORMAT_VideoInfo2)))
	{
		return E_FAIL;
	}

	// we can support this connection
	return S_OK;
}



//
// Verifies that the input and output pins support the media type. 
//
HRESULT
VideoResizer_Filter::CheckTransform (const CMediaType* inputMediaType, const CMediaType* outputMediaType)
{
	HRESULT result = S_OK;

	// test input media
	if (inputMediaType != 0)
	{
		result = this->CheckInputType (inputMediaType);
		if (result != S_OK)
		{
			return result;
		}
	}

	// test output media
	if (outputMediaType != 0)
	{
		result = this->CheckOutputType (outputMediaType);
		if (result != S_OK)
		{
			return result;
		}
	}
	// this transform supported
	return S_OK;
}



//
// Sets the number and size of buffers required for the transfer. 
//
HRESULT VideoResizer_Filter::DecideBufferSize ( IMemAllocator* allocator, ALLOCATOR_PROPERTIES* properties )
{
	// make sure the input pin is connected
	if (this->m_pInput->IsConnected () == FALSE)
	{
		return E_UNEXPECTED;
	}

	// assertions
	CheckPointer (allocator, E_POINTER);
	CheckPointer (properties, E_POINTER);

	properties->cBuffers	= 1;

	properties->cbBuffer	= output_width*output_height*2;

	// set properties
	ALLOCATOR_PROPERTIES actualProperties;
	HRESULT result = allocator->SetProperties (properties, &actualProperties);
	if (result != S_OK)
	{
		return result;
	}

	// verify we got enough stuff
	if ((actualProperties.cBuffers < properties->cBuffers) ||
			(actualProperties.cbBuffer < properties->cbBuffer))
	{
		return E_FAIL;
	}

	return S_OK;
}


//
// Returns one of the media types that the output pin supports 
//
HRESULT VideoResizer_Filter::GetMediaType ( int position, CMediaType* mediaType )
{
	// make sure the input pin is connected
	if (this->m_pInput->IsConnected () == FALSE)
	{
		return E_UNEXPECTED;
	}

	// assertion
	if (mediaType == 0)
	{
		return E_POINTER;
	}
	if (position < 0)
	{
		return E_INVALIDARG;
	}
	if (position > 1)
	{
		return VFW_S_NO_MORE_ITEMS;
	}

	// VIDEOINFOHEADER
	VIDEOINFOHEADER videoInfo;
	ZeroMemory (&videoInfo, sizeof (VIDEOINFOHEADER));
	videoInfo.rcSource.left						= 0;
	videoInfo.rcSource.top						= 0;
	videoInfo.rcSource.right					= output_width;
	videoInfo.rcSource.bottom					= output_height;
	videoInfo.rcTarget.left						= 0;
	videoInfo.rcTarget.top						= 0;
	videoInfo.rcTarget.right					= output_width;
	videoInfo.rcTarget.bottom					= output_height;
	videoInfo.bmiHeader.biSize					= sizeof (BITMAPINFOHEADER);
	videoInfo.bmiHeader.biWidth					= output_width;
	videoInfo.bmiHeader.biHeight				= output_height;
	videoInfo.bmiHeader.biPlanes				= 1;
	videoInfo.bmiHeader.biBitCount				= 16;
	videoInfo.bmiHeader.biSizeImage				= output_width*output_height*2;
	videoInfo.bmiHeader.biCompression			= mmioFOURCC('Y','U','Y','2');

	videoInfo.AvgTimePerFrame = temp_AvgTimePerFrame;

	// initialize media type
	mediaType->InitMediaType ();

	// set major type	
	mediaType->SetType (&MEDIATYPE_Video);

	// Set sub type
	mediaType->SetSubtype (&MEDIASUBTYPE_YUY2);

	// format type VIDEOINFOHEADER
	mediaType->SetFormatType (&FORMAT_VideoInfo);
	if (mediaType->SetFormat (reinterpret_cast<BYTE*> (&videoInfo), sizeof (VIDEOINFOHEADER)) == FALSE)
		return E_FAIL;

	// sample size
	mediaType->SetSampleSize ( videoInfo.bmiHeader.biSizeImage );

	return S_OK;
}


//
// set input media type
//
HRESULT VideoResizer_Filter::SetInputMediaType (const CMediaType* mediaType)
{
	// get image description
	int width			= 0;
	int height			= 0;
	int bitsPerPixel	= 0;

	// VIDEOINFOHEADER
	if (*mediaType->FormatType () == FORMAT_VideoInfo) {
		VIDEOINFOHEADER* header = reinterpret_cast<VIDEOINFOHEADER*> (mediaType->Format ());
		if ( header == 0 ) {
			return E_FAIL;
		}
		width			= (int)(header->bmiHeader.biWidth);
		height			= (int)(header->bmiHeader.biHeight);
		bitsPerPixel	= (int)(header->bmiHeader.biBitCount);
		temp_AvgTimePerFrame = header->AvgTimePerFrame;
	} else if ( *mediaType->FormatType () == FORMAT_VideoInfo2) {
		// VIDEOINFOHEADER2
		VIDEOINFOHEADER2* header = reinterpret_cast<VIDEOINFOHEADER2*> (mediaType->Format ());
		if (header == 0)
		{
			return E_FAIL;
		}
		width			= (int)(header->bmiHeader.biWidth);
		height			= (int)(header->bmiHeader.biHeight);
		bitsPerPixel	= (int)(header->bmiHeader.biBitCount);
		temp_AvgTimePerFrame = header->AvgTimePerFrame;
	} else {	
		// unrecognized format
		return E_FAIL;
	}

	// adjust image information
	if ( bitsPerPixel != 16 || *mediaType->Subtype() != MEDIASUBTYPE_YUY2 )
		return E_FAIL;

//	c_width = c_height = 0;
	input_width = width;
	input_height = height;
	output_width = registry_config.options.width;
	output_height = registry_config.options.height;

	return S_OK;
}


//
// set output media type
//
HRESULT VideoResizer_Filter::SetOutputMediaType (const CMediaType* mediaType)
{
	// copy media type
	this->outputMediaType = *mediaType;

	// setup format info

	// VIDEOINFOHEADER
	if (*mediaType->FormatType () == FORMAT_VideoInfo)
	{
		VIDEOINFOHEADER header;
		ZeroMemory (&header, sizeof (VIDEOINFOHEADER));
		header.bmiHeader.biSize					= sizeof (BITMAPINFOHEADER);
		header.bmiHeader.biWidth				= output_width;
		header.bmiHeader.biHeight				= output_height;
		header.bmiHeader.biPlanes				= 1;
		header.bmiHeader.biBitCount				= 16;
		header.bmiHeader.biCompression			= mmioFOURCC('Y','U','Y','2');
		header.bmiHeader.biSizeImage			= output_width*output_height*2;
		header.AvgTimePerFrame = temp_AvgTimePerFrame;
		this->outputMediaType.SetFormat (reinterpret_cast<BYTE*> (&header), sizeof (VIDEOINFOHEADER));
	}
	// VIDEOINFOHEADER2
	else if (*mediaType->FormatType () == FORMAT_VideoInfo2)
	{
		VIDEOINFOHEADER2 header;
		ZeroMemory (&header, sizeof (VIDEOINFOHEADER2));
		header.bmiHeader.biSize					= sizeof (BITMAPINFOHEADER);
		header.bmiHeader.biWidth				= output_width;
		header.bmiHeader.biHeight				= output_height;
		header.bmiHeader.biPlanes				= 1;
		header.bmiHeader.biBitCount				= 16;
		header.bmiHeader.biCompression			= mmioFOURCC('Y','U','Y','2');
		header.bmiHeader.biSizeImage			= output_width*output_height*2;
		header.AvgTimePerFrame = temp_AvgTimePerFrame;
		this->outputMediaType.SetFormat (reinterpret_cast<BYTE*> (&header), sizeof (VIDEOINFOHEADER2));
	}

	// unrecognized type
	else
	{
		return E_FAIL;
	}

	return S_OK;
}

//
// Informs when the media type is established for the connection. 
//
HRESULT VideoResizer_Filter::SetMediaType (PIN_DIRECTION direction, const CMediaType* mediaType)
{
	// assertion
	CheckPointer (mediaType, E_POINTER);

	// super
	HRESULT result = this->CTransformFilter::SetMediaType (direction, mediaType);
	if (result != S_OK)
	{
		return result;
	}

	// set media type for input pin?
	if (direction == PINDIR_INPUT)
	{
		this->SetInputMediaType (mediaType);

	}

	// set media type for the output pin
	else if (direction == PINDIR_OUTPUT)
	{
		this->SetOutputMediaType (mediaType);
	}

	// unrecognized pin direction
	else
	{
		return E_FAIL;
	}

	return S_OK;
}

bool ExtractBIH(const AM_MEDIA_TYPE* pmt, BITMAPINFOHEADER* bih)
{
	if(pmt)
	{
		if(pmt->formattype == FORMAT_VideoInfo)
		{
			VIDEOINFOHEADER* vih = (VIDEOINFOHEADER*)pmt->pbFormat;
			memcpy(bih, &vih->bmiHeader, sizeof(BITMAPINFOHEADER));
		}
		else if(pmt->formattype == FORMAT_VideoInfo2)
		{
			VIDEOINFOHEADER2* vih = (VIDEOINFOHEADER2*)pmt->pbFormat;
			memcpy(bih, &vih->bmiHeader, sizeof(BITMAPINFOHEADER));
		}

		return(true);
	}
	
	return(false);
}

bool ExtractBIH( IMediaSample* pMS, BITMAPINFOHEADER* bih)
{
	AM_MEDIA_TYPE* pmt = NULL;
	pMS->GetMediaType(&pmt);
	if(pmt)
	{
		bool fRet = ExtractBIH(pmt, bih);

		DeleteMediaType(pmt);

		return(fRet);
	}
	
	return(false);
}

//
// Performs transform operations of the filter. 
//
HRESULT
VideoResizer_Filter::Transform (IMediaSample* inSample,  IMediaSample* outSample)
{
	double	temp_double;
	int		temp_iheight, temp_height, temp_iwidth, top, bottom;
	// perpare output sample
	HRESULT result = this->PrepareOutputSample (inSample, outSample);
	if (result != S_OK)
	{
		return result;
	}
	
	BITMAPINFOHEADER	m_bihOut;

	ExtractBIH( outSample, &m_bihOut );
	
	if ( m_bihOut.biBitCount != 16 ) return S_OK;

	// at this point we can do the transform
	
	BYTE* srcBuffer = 0;
	BYTE* dstBuffer = 0;
	inSample->GetPointer (&srcBuffer);
	outSample->GetPointer (&dstBuffer);

	YUY2_Frame src, dst;

	src.width = input_width;
	src.pitch = input_width*2;
	src.rowsize = input_width*2;

	temp_iheight = input_height;

	dst.data = (char*)dstBuffer;
	dst.width = m_bihOut.biWidth;
	dst.pitch = m_bihOut.biWidth*2;
	dst.rowsize = m_bihOut.biWidth*2;

	temp_height = m_bihOut.biHeight;

	if ( registry_config.options.crop ) {
		if ( temp_iheight-(registry_config.options.crop_top+registry_config.options.crop_bottom) > 0 ) {
			if ( temp_iheight-registry_config.options.crop_top > 0 && temp_iheight-registry_config.options.crop_bottom > 0 ) {
				if ( registry_config.options.flip ) {
					srcBuffer = (unsigned char*)srcBuffer+src.pitch*registry_config.options.crop_bottom;
					temp_iheight -= registry_config.options.crop_top+registry_config.options.crop_bottom;
				} else {
					srcBuffer = (unsigned char*)srcBuffer+src.pitch*registry_config.options.crop_top;
					temp_iheight -= registry_config.options.crop_top+registry_config.options.crop_bottom;
				}
			}
		}
		if ( temp_iheight <= 16 || temp_iheight > input_height ) temp_iheight = input_height;
	}

	if ( registry_config.options.keep_aspect&0x80 ) {
		temp_iwidth = input_width;

		if ( registry_config.options.vcdfix && (input_width == 352 || input_width == 480 || input_width == 720) ) {
			if ( input_height == 240 ) {
				temp_iwidth = 320;
			}
			if ( input_height == 288 ) {
				temp_iwidth = 384;
			}
			if ( input_height == 480 ) {
				temp_iwidth = 640;
			}
			if ( input_height == 576 ) {
				temp_iwidth = 768;
			}
		}

		switch ( registry_config.options.keep_aspect&0x7f ) {
		case 1:
			temp_double = (double)temp_height/(temp_iwidth*9/16);
			break;
		default:
			temp_double = (double)temp_height/(temp_iwidth*3/4);
			break;
		}

		temp_height = temp_iheight*temp_double;

		if ( temp_height > m_bihOut.biHeight )
			temp_height = m_bihOut.biHeight;

		top = (m_bihOut.biHeight-temp_height)/2;
		bottom = (m_bihOut.biHeight-temp_height) - top;

		dst.data = (char*)dstBuffer+dst.pitch*top;

		LetterBox_YUY2( (char*)dstBuffer, dst.pitch, m_bihOut.biHeight, top, bottom );
	}

	src.data = (char*)srcBuffer;
	src.height = temp_iheight;
	dst.height = temp_height;

	Run_YUY2_Resize( &src, &dst, registry_config.options.flip, registry_config.options.b, registry_config.options.c );

	// set output datalength
	outSample->SetActualDataLength ( m_bihOut.biSizeImage );

	return S_OK;
}


//
// Prepares the output sample
//
HRESULT VideoResizer_Filter::PrepareOutputSample (IMediaSample* inSample, IMediaSample* outSample)
{
	// assertions
	CheckPointer (inSample, E_POINTER);
	CheckPointer (outSample, E_POINTER);	

	// set media type
	outSample->SetMediaType (&this->outputMediaType);


	// copy the sample times
	REFERENCE_TIME timeStart = 0;
	REFERENCE_TIME timeEnd = 0;
	if (inSample->GetTime (&timeStart, &timeEnd) == S_OK)
	{
		outSample->SetTime (&timeStart, &timeEnd);
	}

	LONGLONG mediaStart = 0;
	LONGLONG mediaEnd = 0;
	if (inSample->GetMediaTime (&mediaStart, &mediaEnd) == S_OK)
	{
			outSample->SetMediaTime (&mediaStart,&mediaEnd);
	}

	// copy the sync point property
	HRESULT result = inSample->IsSyncPoint ();
	if (result == S_OK)
	{
		outSample->SetSyncPoint (TRUE);
	}
	else if (result == S_FALSE)
	{
		outSample->SetSyncPoint (FALSE);
	}
	else
	{	// an unexpected error has occured...
		return E_UNEXPECTED;
	}

	// copy the preroll property
	result = inSample->IsPreroll ();
	if (result == S_OK)
	{
		outSample->SetPreroll (TRUE);
	}
	else if (result == S_FALSE)
	{
		outSample->SetPreroll (FALSE);
	}
	else
	{	// an unexpected error has occured...
		return E_UNEXPECTED;
	}

	// copy the discontinuity property
	result = inSample->IsDiscontinuity ();
	if (result == S_OK)
	{
		outSample->SetDiscontinuity (TRUE);
	}
	else if (result == S_FALSE)
	{
		outSample->SetDiscontinuity(FALSE);
	}
	else
	{	// an unexpected error has occured...
		return E_UNEXPECTED;
	}


	return S_OK;

} // VideoResizer_Filter::PrepareOutputSample

//
// Fills a counted array of GUID values where each GUID specifies the CLSID of each property page 
// that can be displayed in the property sheet for this object.
//
STDMETHODIMP VideoResizer_Filter::GetPages (CAUUID* pages)
{
	CAutoLock autoLock (&this->m_csFilter);

	CheckPointer (pages, E_POINTER);

	pages->cElems = 1;
	pages->pElems = (GUID*)(CoTaskMemAlloc (sizeof (GUID)));
	if (pages->pElems == 0)
	{
		return E_OUTOFMEMORY;
	}

	*pages->pElems = CLSID_About_PropertyPage;

	return S_OK;
}
